@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-# Copyright (C) 2019-2020 Benjamin Drung <bdrung@posteo.de>
+# Copyright (C) 2019-2021 Benjamin Drung <bdrung@posteo.de>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -68,7 +68,14 @@ class Mmdebstrap:
     def construct_parameters(self, output_dir, simulate=False):
         """Construct the parameter for mmdebstrap from a given dictionary."""
         # pylint: disable=too-many-branches
-        cmd = ["mmdebstrap", "-v"]
+        cmd = ["mmdebstrap"]
+        log_level = self.logger.getEffectiveLevel()
+        if log_level >= logging.ERROR:
+            cmd += ["-q"]
+        elif log_level <= logging.DEBUG:
+            cmd += ["--debug"]
+        elif log_level <= logging.INFO:
+            cmd += ["-v"]
         if simulate:
             cmd += ["--simulate"]
         mmdebstrap = self.config.get("mmdebstrap", {})
@@ -126,13 +133,22 @@ class Mmdebstrap:
         cmd = self.construct_parameters(output_dir, simulate)
         self.logger.info("Calling %s", escape_cmd(cmd))
         subprocess.check_call(cmd)
+        self.clamp_mtime(output_dir)
 
-        for file in (os.path.join(output_dir, "manifest"),
-                     self.config.get("mmdebstrap", {}).get("target", "")):
-            if os.path.isfile(file):
-                clamp_mtime(file, self.config.source_date_epoch)
-
-        clamp_mtime(output_dir, self.config.source_date_epoch)
+    def clamp_mtime(self, output_dir):
+        """Clamp the modification time of the manifest, target, and output directory."""
+        for path in (
+            os.path.join(output_dir, "manifest"),
+            self.config.get("mmdebstrap", {}).get("target", ""),
+            output_dir,
+        ):
+            if os.path.exists(path):
+                try:
+                    clamp_mtime(path, self.config.source_date_epoch)
+                except OSError as error:
+                    self.logger.error(
+                        "Failed to change modification time of '%s': %s", path, error
+                    )
 
 
 def clamp_mtime(path, source_date_epoch):
@@ -187,7 +203,6 @@ def sanitize_list(list_):
 def parse_args(args):  # pylint: disable=too-many-statements
     """Parse the given command line arguments."""
     parser = argparse.ArgumentParser(description=__doc__)
-    # parser.add_argument("-t", "--target", help="path to a tarball filename or a directory")
     # parser.add_argument("-m", "--manifest", help="Store packages manifest in given file")
     parser.add_argument(
         "-c", "--config", action="append", default=[], help="bdebstrap configuration YAML."
@@ -212,6 +227,27 @@ def parse_args(args):  # pylint: disable=too-many-statements
     )
     parser.add_argument("-b", "--output-base-dir", default=".", help="output base directory")
     parser.add_argument("-o", "--output", help="output directory (default: output-base-dir/name)")
+    parser.add_argument(
+        "-q",
+        "--quiet",
+        "--silent",
+        dest="log_level",
+        help="Do not write anything to standard error except errors.",
+        action="store_const",
+        const=logging.ERROR,
+        default=logging.WARNING,
+    )
+    parser.add_argument(
+        "-v",
+        "--verbose",
+        dest="log_level",
+        help=(
+            "Write informational messages to standard error. Instead of progress bars, "
+            "mmdebstrap writes the dpkg and apt output directly to standard error."
+        ),
+        action="store_const",
+        const=logging.INFO,
+    )
     parser.add_argument(
         "--debug",
         dest="log_level",
@@ -221,15 +257,6 @@ def parse_args(args):  # pylint: disable=too-many-statements
         ),
         action="store_const",
         const=logging.DEBUG,
-        default=logging.INFO,
-    )
-    parser.add_argument(
-        "-q",
-        "--quiet",
-        dest="log_level",
-        help="Decrease output verbosity to warnings and errors",
-        action="store_const",
-        const=logging.WARNING,
     )
     parser.add_argument(
         "-f",
@@ -286,7 +313,7 @@ def parse_args(args):  # pylint: disable=too-many-statements
         "--dpkgopt", action="append", help="Pass arbitrary options or configuration files to dpkg."
     )
     parser.add_argument(
-        "--hostname", help="Write the given HOSTNAME into /etc/hostname in the target chroot.",
+        "--hostname", help="Write the given HOSTNAME into /etc/hostname in the target chroot."
     )
     parser.add_argument(
         "--install-recommends",
@@ -474,11 +501,17 @@ def dict_merge(this, other):
     :return: None
     """
     for key in other.keys():
-        if (key in this and isinstance(this[key], collections.abc.MutableMapping)
-                and isinstance(other[key], collections.abc.Mapping)):
+        if (
+            key in this
+            and isinstance(this[key], collections.abc.MutableMapping)
+            and isinstance(other[key], collections.abc.Mapping)
+        ):
             dict_merge(this[key], other[key])
-        elif (key in this and isinstance(this[key], collections.abc.MutableSequence)
-              and isinstance(other[key], collections.abc.Sequence)):
+        elif (
+            key in this
+            and isinstance(this[key], collections.abc.MutableSequence)
+            and isinstance(other[key], collections.abc.Sequence)
+        ):
             this[key] += other[key]
         else:
             this[key] = other[key]
@@ -720,16 +753,16 @@ def main(argv):
         config.set_source_date_epoch()
         config.check()
     except OSError:
-        sys.exit(1)
+        return 1
     except ValueError as error:
         logger.error("%s", error)
-        sys.exit(1)
+        return 1
 
     if not args.output:
         args.output = os.path.join(args.output_base_dir, config["name"])
 
     if not prepare_output_dir(args.output, args.force, args.simulate):
-        sys.exit(1)
+        return 1
     config.save(os.path.join(args.output, "config.yaml"), args.simulate)
 
     # Set environment variables
@@ -742,7 +775,7 @@ def main(argv):
 
     mmdebstrap = config.get("mmdebstrap", {})
 
-    if "target" in mmdebstrap and "/" not in mmdebstrap["target"]:
+    if mmdebstrap.get("target") not in (None, "-") and "/" not in mmdebstrap["target"]:
         mmdebstrap["target"] = os.path.join(args.output, mmdebstrap["target"])
 
     if "LD_PRELOAD" in os.environ:
@@ -756,11 +789,15 @@ def main(argv):
         logger.error(
             "mmdebstrap failed with exit code %i. See above for details.", error.returncode
         )
-        sys.exit(1)
+        return 1
 
-    logger.info("Build successful in '%s'.", mmdebstrap["target"])
+    if mmdebstrap.get("target") in (None, "-"):
+        logger.info("Build successful and sent uncompressed tarball to standard output.")
+    else:
+        logger.info("Build successful in '%s'.", mmdebstrap["target"])
     logger.info("Execution time: %s", duration_str(time.time() - start_time))
+    return 0
 
 
 if __name__ == "__main__":
-    main(sys.argv[1:])
+    sys.exit(main(sys.argv[1:]))  # pragma: no cover
