@@ -1,4 +1,4 @@
-# Copyright (C) 2019 Benjamin Drung <bdrung@posteo.de>
+# Copyright (C) 2019-2021 Benjamin Drung <bdrung@posteo.de>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -29,6 +29,11 @@ class TestArguments(unittest.TestCase):
     This unittest class tests the argument parsing.
     """
 
+    def test_debug(self):
+        """Test --debug argument parsing."""
+        args = parse_args(["--debug"])
+        self.assertEqual(args.log_level, logging.DEBUG)
+
     def test_empty_args(self):
         """Test setting arguments to empty strings."""
         args = parse_args(
@@ -84,7 +89,7 @@ class TestArguments(unittest.TestCase):
                 "hostname": None,
                 "install_recommends": False,
                 "keyring": None,
-                "log_level": logging.INFO,
+                "log_level": logging.WARNING,
                 "mirrors": [],
                 "mode": None,
                 "name": None,
@@ -246,7 +251,6 @@ class TestConfig(unittest.TestCase):
             config,
             {
                 "mmdebstrap": {
-                    "architectures": ["amd64"],
                     "keyrings": ["/usr/share/keyrings/debian-archive-keyring.gpg"],
                     "mode": "unshare",
                     "suite": "unstable",
@@ -296,7 +300,7 @@ class TestConfig(unittest.TestCase):
             {
                 "mmdebstrap": {
                     "aptopts": ['Apt::Install-Recommends "0"'],
-                    "architectures": ["amd64", "i386"],
+                    "architectures": ["i386"],
                     "components": ["main", "non-free"],
                     "dpkgopts": ["force-confdef", "force-confold"],
                     "keyrings": [
@@ -351,7 +355,6 @@ class TestConfig(unittest.TestCase):
             config,
             {
                 "mmdebstrap": {
-                    "architectures": ["amd64"],
                     "keyrings": ["/usr/share/keyrings/debian-archive-keyring.gpg"],
                     "mode": "unshare",
                     "suite": "unstable",
